/******************************************************************************
** $Id: ua_security_layer.c,v 1.2 2007/02/08 12:16:59 gergap Exp $
**
** Copyright (C) 2006-2007 ascolab GmbH. All Rights Reserved.
** Web: http://www.ascolab.com
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
** 
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
** 
** Project: OpcUa Wireshark Plugin
**
** Description: OpcUa Security Layer Decoder.
**
** Author: Gerhard Gappmeier <gerhard.gappmeier@ascolab.com>
** Last change by: $Author: gergap $
**
******************************************************************************/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gmodule.h>
#include <epan/packet.h>
#include "ua_application_layer.h"

static int hf_opcua_security_sigtoklen = -1;
static int hf_opcua_security_sigtok = -1;
static int hf_opcua_security_dummy = -1;

static hf_register_info hf[] =
{
    { &hf_opcua_security_sigtoklen,
    {  "SigningTokenLength",  "security.sigtoklen", FT_UINT32, BASE_DEC,  NULL, 0x0,    "",    HFILL }
    },
    { &hf_opcua_security_sigtok,
    {  "SigningToken",        "security.sigtok",    FT_BYTES,  BASE_HEX,  NULL, 0x0,    "",    HFILL }
    },
    { &hf_opcua_security_dummy,
    {  "Security Dummy Data",        "security.dummy",    FT_BYTES,  BASE_HEX,  NULL, 0x0,    "",    HFILL }
    }
};

/** Register security layer types. */
void registerSecurityLayerTypes(int proto)
{
    proto_register_field_array(proto, hf, array_length(hf));
}


/* Security Layer: message parsers */
void parseSecurityLayer(proto_tree *tree, tvbuff_t *tvb, gint *pOffset)
{
    gint32 iLen;

    /* hack: we skip the dummy security data for now */
    iLen = tvb_get_letohl(tvb, *pOffset);
    iLen += 4;
    proto_tree_add_item(tree, hf_opcua_security_dummy, tvb, *pOffset, iLen, TRUE);    
    *pOffset += iLen;

#if 0
    gint32 iMessageLen, iSigTokLen;
    /*proto_item *ti = NULL;
    proto_tree *security_tree = NULL;*/
    /* message data is transmitted as byte string */
    iMessageLen = tvb_get_letohl(tvb, *pOffset); /* length of byte string */
    *pOffset+=4;


    /* parse signing token is transmitted as byte string */
    iSigTokLen = tvb_get_letohl(tvb, *pOffset); /* length of byte string */
    proto_tree_add_item(tree, hf_opcua_security_sigtoklen, tvb, *pOffset, 4, TRUE);
    *pOffset+=4;
    proto_tree_add_bytes(tree, hf_opcua_security_sigtok, tvb, *pOffset, iSigTokLen, &tvb->real_data[*pOffset]); /* byte string data */
    *pOffset+=iSigTokLen;

    /*ti = proto_tree_add_item(tree, proto_opcua, tvb, 0, -1, FALSE);
    security_tree = proto_item_add_subtree(ti, ett_opcua_transport);*/
#endif
}

